#include <bits/stdc++.h>
using namespace std;

#define TASK "messages"

struct node {
    int v, t;

    node() {}
    node(int v, int t) : v(v), t(t) {}

    bool operator==(node const& rhs) const {
        return v == rhs.v && t == rhs.t;
    }
};

struct edge {
    node v, u;
    int flow, cap;

    edge(node v, node u, int flow, int cap)
        : v(v), u(u), flow(flow), cap(cap) {}

    node go(node const& x) {
        return x == v ? u : v;
    }
};

struct graph {
    static const int INF = (int)1e9;

    vector <edge> edges;
    vector <vector <vector <int> > > g;

    graph(int n, int time)
        : g(n, vector <vector <int> >(time)) {}

    void add_edge(node const& u, node const& v, int cap) {
        get(u).push_back(edges.size());
        edges.push_back(edge(u, v, 0, cap));
        get(v).push_back(edges.size());
        edges.push_back(edge(v, u, 0, 0));
    }

    vector<int>& get(node const& idx) {
        return g[idx.v][idx.t];
    }

    vector < vector <int> > d;
    bool bfs(node const& s, node const& t) {
        int n = (int)g.size();
        int time = (int)g.size();
        d.assign(n, vector <int>(time, -1));

        d[s.v][s.t] = 0;
        queue <node> q;
        q.push(s);
        while (!q.empty()) {
            node v = q.front();
            q.pop();

            for (int eid = 0; eid < (int)get(v).size(); eid++) {
                edge e = edges[get(v)[eid]];
                node to = e.go(v);
                if (e.flow < e.cap && d[to.v][to.t] == -1) {
                    d[to.v][to.t] = d[v.v][v.t] + 1;
                    q.push(to);
                }
            }
        }

        return d[t.v][t.t] != -1;
    }

    vector <vector <int> > p;
    vector <vector <int> > ptr;
    node dest;
    int dfs(node const& v, int flow = INF) {
        for (int &eid = ptr[v.v][v.t]; eid < (int)get(v).size(); eid++) {
            edge &e = edges[get(v)[eid]];
            node to = e.go(v);
            if (d[to.v][to.t] != d[v.v][v.t] + 1) continue;
            if (p[to.v][to.t] == -1 && e.flow < e.cap) {
                p[to.v][to.t] = get(v)[eid];
                int new_flow = dfs(to, min(flow, e.cap - e.flow));
                if (new_flow > 0) {
                    e.flow += new_flow;
                    edges[get(v)[eid] ^ 1].flow -= new_flow;
                    return new_flow;
                }
            }
        }

        if (v == dest) return flow;
        else return 0;
    }

    int max_flow(node const& s, node const& t) {
        int n = (int)g.size();
        int time = (int)g[0].size();
        int flow = 0;

        dest = t;
        while (bfs(s, t)) {
            ptr.assign(n, vector <int>(time));

            while (true) {
                p.assign(n, vector <int>(time, -1));
                int new_flow = dfs(s);
                if (new_flow == 0) break;
                flow += new_flow;
            }
        }

        return flow;
    }
};

bool check(vector <vector <int> > const& g,
           int time, vector <int> const& term) {
    int n = (int)g.size();
    graph g_new(n + 1, time + 1);

    for (int v = 0; v < n; v++) {
        for (int t = 0; t < time; t++) {
            node v_new(v, t);
            for (int cid = 0; cid < (int)g[v].size(); cid++) {
                int to = g[v][cid];
                node to_new(to, t + 1);
                g_new.add_edge(v_new, to_new, 1);
            }
            g_new.add_edge(v_new, node(v, t + 1), n);
        }
    }

    node last(n, time);
    for (int i = 0; i < (int)term.size(); i++) {
        node cur(term[i], time);
        g_new.add_edge(cur, last, 1);
    }

    return g_new.max_flow(node(0, 0), last) == (int)term.size();
}

void solve() {
    int n, m;
    cin >> n >> m;
    vector <vector <int> > g(n);
    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;
        --u; --v;
        g[u].push_back(v);
        g[v].push_back(u);
    }

    int k;
    cin >> k;
    vector <int> term(k);
    for (int i = 0; i < k; i++) {
        cin >> term[i];
        --term[i];
    }

    int l = -1;
    int r = n + 1;
    while (r - l > 1) {
        int m = (l + r) / 2;
        if (check(g, m, term)) r = m;
        else l = m;
    }

    cout << r << endl;
}

int main() {
    freopen(TASK ".in", "r", stdin);
    freopen(TASK ".out", "w", stdout);
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    solve();

    return 0;
}
