import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

public class TestsGen {
    public static final String ENDL = "\r\n";
    int testCount;
    Random random = new Random(1234123);

    void write(String s) {
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(String.format("../tests/%02d", ++testCount));
            System.out.println(testCount);
            pw.println(s);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (pw != null) {
                pw.close();
            }
        }
    }

    void genSamples() throws IOException {
        write("5 4" + ENDL
                + "1 2" + ENDL
                + "2 3" + ENDL
                + "2 4" + ENDL
                + "2 5" + ENDL
                + "3" + ENDL
                + "3 4 5");

        write("9 10" + ENDL
                + "1 2" + ENDL
                + "1 3" + ENDL
                + "2 4" + ENDL
                + "3 4" + ENDL
                + "4 5" + ENDL
                + "4 6" + ENDL
                + "5 6" + ENDL
                + "6 7" + ENDL
                + "6 8" + ENDL
                + "6 9" + ENDL
                + "3" + ENDL
                + "7 8 9");

    }

    int[] genRandomPermutation(int n) {
        List<Integer> q = new ArrayList<>();
        for (int i = 0; i < n; i++) {
            q.add(i);
        }
        Collections.shuffle(q);
        int[] a = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = q.get(i);
        }

        return a;
    }

    void genTree(int n, int[] u, int[] v) {
        int[] q = genRandomPermutation(n);
        for (int i = 1; i < n; i++) {
            u[i - 1] = q[random.nextInt(i)];
            v[i - 1] = q[i];
        }
    }

    static final class Pair {
        int a, b;

        Pair(int a, int b) {
            this.a = a;
            this.b = b;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Pair pair = (Pair) o;

            return a == pair.a && b == pair.b;

        }

        @Override
        public int hashCode() {
            int result = a;
            result = 31 * result + b;
            return result;
        }
    }

    int[] genA(int n, int k) {
        int[] p = genRandomPermutation(n - 1);
        int[] res = new int[k];
        for (int i = 0; i < k; i++) {
            res[i] = p[i] + 1;
        }

        return res;
    }

    void writeAnswer(int n, int m, int[] u, int[] v, int k, int[] a) {
        StringBuilder builder = new StringBuilder();
        builder.append(n).append(" ").append(m).append(ENDL);
        for (int i = 0; i < m; i++) {
            builder.append(u[i] + 1).append(" ").append(v[i] + 1);
            builder.append(ENDL);
        }

        builder.append(k).append(ENDL);
        for (int i = 0; i < k; i++) {
            builder.append(a[i] + 1);
            if (i < k - 1) {
                builder.append(" ");
            }
        }

        write(builder.toString());
    }

    void writeGraph(int n, int m, int k) {
        int[] u = new int[m];
        int[] v = new int[m];
        genGraph(n, m, u, v);
        int[] a = genA(n, k);
        writeAnswer(n, m, u, v, k, a);
    }

    void writeTree(int n, int k) {
        int[] u = new int[n - 1];
        int[] v = new int[n - 1];
        genTree(n, u, v);
        int[] a = genA(n, k);
        writeAnswer(n, n - 1, u, v, k, a);
    }

    void genGraph(int n, int m, int[] u, int[] v) {
        genTree(n, u, v);

        Set<Pair> q = new HashSet<>();
        for (int i = 0; i < n; i++) {
            for (int j = i + 1; j < n; j++) {
                q.add(new Pair(i, j));
            }
        }

        for (int i = 0; i < n - 1; i++) {
            q.remove(new Pair(Math.min(u[i], v[i]), Math.max(u[i], v[i])));
        }

        for (int i = n - 1; i < m; i++) {
            int pos = random.nextInt(q.size());
            Iterator<Pair> it = q.iterator();
            Pair x = null;
            for (int j = 0; j < pos; j++) {
                x = it.next();
            }
            it.remove();
            if (x != null) {
                u[i] = x.a;
                v[i] = x.b;
            }
        }
    }

    void run() throws IOException {
        new File("../tests").mkdir();

        genSamples();

        writeTree(10, 2);
        writeTree(10, 3);
        writeTree(10, 4);
        writeTree(10, 5);
        writeGraph(10, 20, 2);
        writeGraph(10, 20, 3);
        writeGraph(10, 20, 4);
        writeGraph(10, 20, 5);
        writeTree(20, 10);
        writeTree(50, 10);
        writeTree(100, 10);
        writeTree(100, 20);
        writeTree(100, 30);
        writeTree(100, 40);
        writeGraph(20, 50, 10);
        writeGraph(50, 100, 10);
        writeGraph(100, 100, 10);
        writeGraph(50, 100, 40);
        writeGraph(50, 100, 40);
        writeGraph(50, 100, 40);
        writeGraph(100, 100, 80);
        writeGraph(100, 100, 60);
        writeGraph(100, 100, 70);
        writeGraph(100, 100, 50);
        writeGraph(100, 100, 30);
        writeGraph(100, 100, 40);
        writeGraph(100, 100, 90);
        writeGraph(100, 100, 99);
        writeGraph(100, 100, 99);
        writeGraph(100, 100, 99);
        writeGraph(100, 100, 99);
    }

    public static void main(String[] args) throws IOException {
        new TestsGen().run();
    }
}
