import java.io.*;
import java.util.*;

public class Validate {
    private static final int MAX_N = 500;
    private static final int MAX_COORDINATE = 1000_000_000;

    class Point implements Comparable <Point> {
        int x, y, id;

        Point(int x, int y, int id) {
            this.x = x;
            this.y = y;
            this.id = id;
        }

        public int compareTo(Point p) {
            if (x != p.x)
                return x - p.x;
            return y - p.y;
        }

        boolean equals(Point p) {
            return x == p.x && y == p.y;
        }
    };

    public void run() {
        StrictScanner inf = new StrictScanner(System.in);
        int n = inf.nextInt();
        inf.nextLine();
        ensureLimits(n, 1, MAX_N, "Wrong n");

        Point[] ps = new Point[n];
        for (int i = 0; i < n; i++) {
            int x = inf.nextInt();
            int y = inf.nextInt();
            inf.nextLine();
            ensureLimits(x, -MAX_COORDINATE, MAX_COORDINATE, String.format("Wrong x in line %s - ", i + 2));
            ensureLimits(y, -MAX_COORDINATE, MAX_COORDINATE, String.format("Wrong y in line %s - ", i + 2));
            ps[i] = new Point(x, y, i);
        }
        Arrays.sort(ps);
        
        for (int i = 0; i < ps.length - 1; i++) {
            ensure(!ps[i].equals(ps[i + 1]), 
                   String.format("Point number %s is equal to point number %s", ps[i].id + 1, ps[i + 1].id + 1)
                   );
        }

        for (int i = 0; i < ps.length; i++)
        for (int j = i + 1; j < ps.length; j++)
        for (int k = j + 1; k < ps.length; k++) {
            ensure(!onLine(ps[i], ps[j], ps[k]), 
                   String.format("Points number %s, %s and %s lies on one line", ps[i].id + 1, ps[j].id + 1, ps[k].id + 1)
                   );
        }

        for (int i = 1; i <= n; i++) {
            int v = inf.nextInt();
            int u = inf.nextInt();
            inf.nextLine();
            ensureLimits(v, 1, n, String.format("First child of vertex %s is wrong, line %s,", i + 1, i + n + 2));
            ensureLimits(u, 1, n, String.format("Second child of vertex %s is wrong, line %s,", i + 1, i + n + 2));
            ensure(v != u, String.format("Children of vertex %s are equal, line %s", i + 1, i + n + 2));
            ensure(v != i && v != u, String.format("Some children of vertex %s are equal to itself, line %s", i + 1, i + n + 2));
        }
        inf.close();
    }

    boolean onLine(Point a, Point b, Point c) {
        long A = b.y - a.y;
        long B = a.x - b.x;
        long C = -a.x * A - a.y * B;
        
        return c.x * A + c.y * B + C == 0L;
    }

    public static void main(String[] args) {
        new Validate().run();
    }

    public class StrictScanner {
        private final BufferedReader in;
        private String line = "";
        private int pos;
        private int lineNo;

        public StrictScanner(InputStream source) {
            in = new BufferedReader(new InputStreamReader(source));
            nextLine();
        }

        public void close() {
            ensure(line == null, "Extra data at the end of file");
            try {
                in.close();
            } catch (IOException e) {
                throw new AssertionError("Failed to close with " + e);
            }
        }

        public void nextLine() {
            ensure(line != null, "EOF");
            ensure(pos == line.length(),  "Extra characters on line " + lineNo);
            try {
                line = in.readLine();
            } catch (IOException e) {
                throw new AssertionError("Failed to read line with " + e);
            }
            pos = 0;
            lineNo++;
        }

        public String next() {
            ensure(line != null, "EOF");
            ensure(line.length() > 0,  "Empty line " + lineNo);
            if (pos == 0)
                ensure(line.charAt(0) > ' ',  "Line " + lineNo + " starts with whitespace");
            else {
                ensure(pos < line.length(),  "Line " + lineNo + " is over");
                ensure(line.charAt(pos) == ' ',  "Wrong whitespace on line " + lineNo);
                pos++;
                ensure(pos < line.length(),  "Line " + lineNo + " is over");
                ensure(line.charAt(0) > ' ',  "Line " + lineNo + " has double whitespace");
            }
            StringBuilder sb = new StringBuilder();
            while (pos < line.length() && line.charAt(pos) > ' ')
                sb.append(line.charAt(pos++));
            return sb.toString();
        }

        public int nextInt() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',  "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+',  "Extra leading '+' in number " + s + " on line " + lineNo);
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }

        public long nextLong() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Long.parseLong(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public double nextDouble() {
            String s = next();
            ensure(s.length() == 1 || s.startsWith("0.") || s.charAt(0) != '0',  "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+',  "Extra leading '+' in number " + s + " on line " + lineNo);
            try {
                return Double.parseDouble(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }
    }

    void ensure(boolean b, String message) {
        if (!b) {
            throw new AssertionError(message);
        }
    }
    
    void ensureLimits(int n, int from, int to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to " + to);
    }

    void ensureLimits(long n, long from, long to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to " + to);
    }
}