import java.io.*;
import java.math.*;
import java.util.*;

public class TestsGen {
    static final int MX = 50_000;

    int testCount;

    void write(String s) throws IOException {
        PrintWriter pw = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount);
        pw.println(s);
        pw.close();
    }
    
    Random random = new Random(43534543);

    void genSample() throws IOException {
        write("5\r\n1 2 3 4 5\r\n1 2\r\n2 3\r\n3 4\r\n4 5");
        write("4\r\n0 1 1 1\r\n1 2\r\n1 3\r\n1 4");
    }

    int[] genArray(int n) {
        int res[] = new int[n];
        for (int i = 0; i < n; i++) {
            res[i] = random.nextInt(100001);
        }
        return res;
    }

    void write(int n, int[] a, int[] u, int[] v) throws IOException {
        PrintWriter pw = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount);
        pw.println(n);
        for (int i = 0; i < n - 1; i++) {
            pw.print(a[i] + " ");
        }
        pw.println(a[n - 1]);
        int p[] = new int[n - 1];
        for (int i = 0; i < n - 1; i++) {
            p[i] = i;
            int k = random.nextInt(i + 1);
            int o = p[k];
            p[k] = p[i];
            p[i] = o;
            if (random.nextInt(2) == 1) {
                o = u[i];
                u[i] = v[i];
                v[i] = o;
            }
        }
        for (int i = 0; i < n - 1; i++) {
            pw.println(u[p[i]] + " " + v[p[i]]);
        }
        pw.close();
    }

    void genLine(int n) throws IOException {
        int u[] = new int[n], v[] = new int[n];
        for (int i = 0; i < n - 1; i++) {
            u[i] = i + 1;
            v[i] = i + 2;
        }
        write(n, genArray(n), u, v);
    }

    void genTree(int n) throws IOException {
        int u[] = new int[n], v[] = new int[n];
        for (int i = 0; i < n - 1; i++) {
            u[i] = random.nextInt(i + 1) + 1;
            v[i] = i + 2;
        }
        write(n, genArray(n), u, v);
    }

    void genMaxAns(int n) throws IOException {
        int u[] = new int[n], v[] = new int[n], a[] = new int[n];
        for (int i = 0; i < n - 1; i++) {
            u[i] = i + 1;
            v[i] = i + 2;
        }
        for (int i = 0; i < n; i++) {
            a[i] = i + 1;
        }
        write(n, a, u, v);
    }

    void genBinaryTree(int n, boolean f) throws IOException {
        int u[] = new int[n], v[] = new int[n], a[] = new int[n];
        for (int i = 1; i <= n; i++) {
            if (i > 1) {
                u[i - 2] = i / 2;
                v[i - 2] = i;
            }
            a[i - 1] = f ? i : n - i + 1;
        }
        write(n, a, u, v);
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        
        genSample();        
        for (int i = 1; i <= 10; i++) {
            genLine(i);
        }
        for (int i = 1; i <= 10; i++) {
            genTree(i);
        }
        for (int i = 1; 10000 * i <= MX; i++) {
            genLine(10000 * i);
        }
        for (int i = 1; 10000 * i <= MX; i++) {
            genTree(10000 * i);
        }
        for (int i = 4; i >= 0; i--) {
            genLine(MX - i);
            genTree(MX - i);
        }
        genMaxAns(MX);
        genBinaryTree(MX, false);
        genBinaryTree(MX, true);
    }

    public static void main(String[] args) throws IOException {
        new TestsGen().run();    }

}
